==> moonlight-1.20-2.16.14-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/mixins/PistonBlockEntityMixin.java <==
package net.mehvahdjukaar.moonlight.core.mixins;

import net.mehvahdjukaar.moonlight.api.block.IBlockHolder;
import net.mehvahdjukaar.moonlight.api.block.IPistonMotionReact;
import net.mehvahdjukaar.moonlight.core.misc.IExtendedPistonTile;
import net.mehvahdjukaar.moonlight.core.network.ClientBoundOnPistonMovedBlockPacket;
import net.mehvahdjukaar.moonlight.core.network.ModMessages;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_2586;
import net.minecraft.class_259;
import net.minecraft.class_2591;
import net.minecraft.class_2669;
import net.minecraft.class_2680;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_2669.class)
public abstract class PistonBlockEntityMixin extends class_2586 implements IBlockHolder, IExtendedPistonTile {

    @Shadow
    private class_2350 direction;
    @Shadow
    private float progress;
    @Shadow
    private float progressO;

    @Shadow
    private class_2680 movedState;

    protected PistonBlockEntityMixin(class_2591<?> pType, class_2338 pWorldPosition, class_2680 pBlockState) {
        super(pType, pWorldPosition, pBlockState);
    }

    @Override
    public class_2680 getHeldBlock() {
        return this.movedState;
    }

    @Override
    public boolean setHeldBlock(class_2680 state) {
        this.movedState = state;
        return true;
    }

    @Shadow
    protected abstract float getExtendedProgress(float pProgress);

    @Shadow
    private boolean extending;

    @Inject(method = "tick", at = @At("TAIL"))
    private static void whileMoving(class_1937 pLevel, class_2338 pPos, class_2680 pState, class_2669 tile, CallbackInfo info) {
        if (tile instanceof IExtendedPistonTile t) {
            t.tickMovedBlock(pLevel, pPos);
        }
    }

    @Override
    public void tickMovedBlock(class_1937 level, class_2338 pos) {
        if (this.progressO < 1.0F) {
            class_2248 b = this.movedState.method_26204();
            if (b instanceof IPistonMotionReact mr && mr.ticksWhileMoved()) {
                class_238 aabb = this.moveByPositionAndProgress(pos, class_259.method_1077().method_1107());
                mr.moveTick(level, pos, this.movedState, aabb, (class_2669) (Object) this);
            }
        }
    }

    @Unique
    private class_238 moveByPositionAndProgress(class_2338 pos, class_238 aabb) {
        double d0 = this.getExtendedProgress(this.progress);
        return aabb.method_989(pos.method_10263() + d0 * this.direction.method_10148(), pos.method_10264() + d0 * this.direction.method_10164(), pos.method_10260() + d0 * this.direction.method_10165());
    }

    @Inject(method = "finalTick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/Level;neighborChanged(Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/Block;Lnet/minecraft/core/BlockPos;)V",
            shift = At.Shift.AFTER))
    public void onFinishedShortPulse(CallbackInfo ci) {
        if (!field_11863.field_9236 && this.movedState.method_26204() instanceof IPistonMotionReact pr) {
            ModMessages.CHANNEL.sendToAllClientPlayersInDefaultRange(field_11863, field_11867,
                    new ClientBoundOnPistonMovedBlockPacket(this.field_11867, this.movedState, this.direction, this.extending));
            pr.onMoved(this.field_11863, this.field_11867, this.movedState, this.direction, this.extending);
        }
    }

    @Inject(method = "tick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/block/Block;updateOrDestroy(Lnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/world/level/LevelAccessor;Lnet/minecraft/core/BlockPos;I)V",
            shift = At.Shift.AFTER), require = 1)
    private static void onFinishedMoving(class_1937 level, class_2338 pos, class_2680 state, class_2669 blockEntity, CallbackInfo ci) {
        class_2680 movedState = blockEntity.method_11495();
        if (!level.field_9236 && movedState.method_26204() instanceof IPistonMotionReact pr) {
            ModMessages.CHANNEL.sendToAllClientPlayersInDefaultRange(level, pos,
                    new ClientBoundOnPistonMovedBlockPacket(pos, movedState,
                            blockEntity.method_11498(), blockEntity.method_11501()));
            //calls on server here and client above
            pr.onMoved(level, blockEntity.method_11016(), movedState, blockEntity.method_11498(), blockEntity.method_11501());
        }
    }

    @Inject(method = "tick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/Level;neighborChanged(Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/Block;Lnet/minecraft/core/BlockPos;)V",
            shift = At.Shift.AFTER), require = 1)
    private static void onFinishedMoving2(class_1937 level, class_2338 pos, class_2680 state, class_2669 blockEntity, CallbackInfo ci) {
        class_2680 movedState = blockEntity.method_11495();
        if (!level.field_9236 && movedState.method_26204() instanceof IPistonMotionReact pr) {
            ModMessages.CHANNEL.sendToAllClientPlayersInDefaultRange(level, pos,
                    new ClientBoundOnPistonMovedBlockPacket(pos, movedState,
                            blockEntity.method_11498(), blockEntity.method_11501()));
            pr.onMoved(level, blockEntity.method_11016(), movedState, blockEntity.method_11498(), blockEntity.method_11501());
        }
    }

}

==> moonlight-1.21-2.25.0-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/mixins/PistonBlockEntityMixin.java <==
package net.mehvahdjukaar.moonlight.core.mixins;

import net.mehvahdjukaar.moonlight.api.block.IBlockHolder;
import net.mehvahdjukaar.moonlight.api.block.IPistonMotionReact;
import net.mehvahdjukaar.moonlight.api.platform.network.NetworkHelper;
import net.mehvahdjukaar.moonlight.core.misc.IExtendedPistonTile;
import net.mehvahdjukaar.moonlight.core.network.ClientBoundOnPistonMovedBlockPacket;
import net.mehvahdjukaar.moonlight.core.network.ModNetworking;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_238;
import net.minecraft.class_2586;
import net.minecraft.class_259;
import net.minecraft.class_2591;
import net.minecraft.class_2669;
import net.minecraft.class_2680;
import net.minecraft.class_3218;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_2669.class)
public abstract class PistonBlockEntityMixin extends class_2586 implements IBlockHolder, IExtendedPistonTile {

    @Shadow
    private class_2350 direction;
    @Shadow
    private float progress;
    @Shadow
    private float progressO;

    @Shadow
    private class_2680 movedState;

    protected PistonBlockEntityMixin(class_2591<?> pType, class_2338 pWorldPosition, class_2680 pBlockState) {
        super(pType, pWorldPosition, pBlockState);
    }

    @Override
    public class_2680 getHeldBlock() {
        return this.movedState;
    }

    @Override
    public boolean setHeldBlock(class_2680 state) {
        this.movedState = state;
        return true;
    }

    @Shadow
    protected abstract float getExtendedProgress(float pProgress);

    @Shadow
    private boolean extending;

    @Inject(method = "tick", at = @At("TAIL"))
    private static void whileMoving(class_1937 pLevel, class_2338 pPos, class_2680 pState, class_2669 tile, CallbackInfo info) {
        if (tile instanceof IExtendedPistonTile t) {
            t.tickMovedBlock(pLevel, pPos);
        }
    }

    @Override
    public void tickMovedBlock(class_1937 level, class_2338 pos) {
        if (this.progressO < 1.0F) {
            class_2248 b = this.movedState.method_26204();
            if (b instanceof IPistonMotionReact mr && mr.ticksWhileMoved()) {
                class_238 aabb = this.moveByPositionAndProgress(pos, class_259.method_1077().method_1107());
                mr.moveTick(level, pos, this.movedState, aabb, (class_2669) (Object) this);
            }
        }
    }

    @Unique
    private class_238 moveByPositionAndProgress(class_2338 pos, class_238 aabb) {
        double d0 = this.getExtendedProgress(this.progress);
        return aabb.method_989(pos.method_10263() + d0 * this.direction.method_10148(), pos.method_10264() + d0 * this.direction.method_10164(), pos.method_10260() + d0 * this.direction.method_10165());
    }

    @Inject(method = "finalTick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/Level;neighborChanged(Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/Block;Lnet/minecraft/core/BlockPos;)V",
            shift = At.Shift.AFTER))
    public void onFinishedShortPulse(CallbackInfo ci) {
        if (!field_11863.field_9236 && this.movedState.method_26204() instanceof IPistonMotionReact pr) {
            NetworkHelper.sendToAllClientPlayersInDefaultRange((class_3218) field_11863, field_11867,
                    new ClientBoundOnPistonMovedBlockPacket(this.field_11867, this.movedState, this.direction, this.extending));
            pr.onMoved(this.field_11863, this.field_11867, this.movedState, this.direction, this.extending);
        }
    }

    @Inject(method = "tick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/block/Block;updateOrDestroy(Lnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/world/level/block/state/BlockState;Lnet/minecraft/world/level/LevelAccessor;Lnet/minecraft/core/BlockPos;I)V",
            shift = At.Shift.AFTER), require = 1)
    private static void onFinishedMoving(class_1937 level, class_2338 pos, class_2680 state, class_2669 blockEntity, CallbackInfo ci) {
        class_2680 movedState = blockEntity.method_11495();
        if (!level.field_9236 && movedState.method_26204() instanceof IPistonMotionReact pr) {
            NetworkHelper.sendToAllClientPlayersInDefaultRange((class_3218) level, pos,
                    new ClientBoundOnPistonMovedBlockPacket(pos, movedState,
                            blockEntity.method_11498(), blockEntity.method_11501()));
            //calls on server here and client above
            pr.onMoved(level, blockEntity.method_11016(), movedState, blockEntity.method_11498(), blockEntity.method_11501());
        }
    }

    @Inject(method = "tick", at = @At(value = "INVOKE", target = "Lnet/minecraft/world/level/Level;neighborChanged(Lnet/minecraft/core/BlockPos;Lnet/minecraft/world/level/block/Block;Lnet/minecraft/core/BlockPos;)V",
            shift = At.Shift.AFTER), require = 1)
    private static void onFinishedMoving2(class_1937 level, class_2338 pos, class_2680 state, class_2669 blockEntity, CallbackInfo ci) {
        class_2680 movedState = blockEntity.method_11495();
        if (!level.field_9236 && movedState.method_26204() instanceof IPistonMotionReact pr) {
            NetworkHelper.sendToAllClientPlayersInDefaultRange((class_3218) level, pos,
                    new ClientBoundOnPistonMovedBlockPacket(pos, movedState,
                            blockEntity.method_11498(), blockEntity.method_11501()));
            pr.onMoved(level, blockEntity.method_11016(), movedState, blockEntity.method_11498(), blockEntity.method_11501());
        }
    }

}
